/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.List;

import org.fortiss.tooling.kernel.extension.ILogMessageHandler;
import org.fortiss.tooling.kernel.extension.data.ILogMessageContext;
import org.fortiss.tooling.kernel.extension.data.LogMessage;
import org.fortiss.tooling.kernel.service.IKernelIntrospectionSystemService;
import org.fortiss.tooling.kernel.service.ILoggingService;
import org.fortiss.tooling.kernel.service.base.ObjectAwareServiceBase;

/**
 * Implements the {@link ILoggingService}. It ensures that the matching handlers are called.
 * 
 * @author diewald
 */
public class LoggingService extends ObjectAwareServiceBase<ILogMessageHandler<LogMessage>>
		implements ILoggingService {
	/** The singleton instance. */
	private static final LoggingService INSTANCE = new LoggingService();

	/** Returns singleton instance of the service. */
	public static LoggingService getInstance() {
		return INSTANCE;
	}

	/** The UI message provider extension point ID. */
	private static final String EXTENSION_POINT_NAME =
			"org.fortiss.tooling.kernel.uiMessageHandler";

	/** The UI message provider configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "uiMessageHandler";

	/** The UI message provider attribue's element name. */
	private static final String ATTRIBUTE_NAME = "handler";

	/** {@inheritDoc} */
	@Override
	public void initializeService() {
		super.initializeService();
	}

	/** {@inheritDoc} */
	@Override
	public void startService() {
		IKernelIntrospectionSystemService.getInstance().registerService(this);
	}

	/** Registers a handler for the given class of {@link LogMessage}s. */
	@Override
	@SuppressWarnings("unchecked")
	public <LM extends LogMessage> void registerLogMessageHandler(ILogMessageHandler<LM> handler,
			Class<LM> logMessageClass) {
		addHandler(logMessageClass, (ILogMessageHandler<LogMessage>)handler);
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionDescription() {
		return "This service allows the registration of logging message handlers.";
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionLabel() {
		return "Logging Service";
	}

	/** {@inheritDoc} */
	@Override
	public void processMessage(LogMessage message, ILogMessageContext context) {
		List<ILogMessageHandler<LogMessage>> handlerList =
				getRegisteredHandlers(message.getClass());
		for(ILogMessageHandler<LogMessage> handler : handlerList) {
			handler.showMessage(message, context);
		}
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return ATTRIBUTE_NAME;
	}
}
