/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.jface;

import org.conqat.lib.commons.assertion.CCSMPre;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * An base class for editing support based on textual values which are
 * validated.
 * 
 * @author hummelb
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @ConQAT.Rating GREEN Hash: 35C0D09FCF5C69B23B17F305B2E4560D
 */
public abstract class ValidatedTextEditingSupport<T> extends EditingSupport {

	/** The type expected for elements. */
	private final Class<T> elementClass;

	/** Constructor. */
	public ValidatedTextEditingSupport(ColumnViewer viewer,
			Class<T> elementClass) {
		super(viewer);
		this.elementClass = elementClass;
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns true.
	 */
	@Override
	protected boolean canEdit(Object element) {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	protected CellEditor getCellEditor(final Object element) {
		CCSMPre.isInstanceOf(element, elementClass);

		// we need to slightly extend the editor to support the display of the
		// decoration
		TextCellEditor editor = new TextCellEditor((Composite) getViewer()
				.getControl()) {

			@Override
			protected Control createControl(Composite parent) {
				Control control = super.createControl(parent);

				final ControlDecoration decoration = new ControlDecoration(
						text, SWT.LEFT | SWT.TOP);
				decoration.setImage(FieldDecorationRegistry.getDefault()
						.getFieldDecoration(FieldDecorationRegistry.DEC_ERROR)
						.getImage());

				text.addModifyListener(new ModifyListener() {
					/** {@inheritDoc} */
					public void modifyText(ModifyEvent e) {
						editOccured(e); // pass edit event further on
						
						if (getErrorMessage() == null) {
							decoration.hide();
						} else {
							decoration.setDescriptionText(getErrorMessage());
							decoration.show();
						}
					}
				});

				return control;
			}
		};

		editor.setValidator(new ICellEditorValidator() {
			@SuppressWarnings("unchecked")
			public String isValid(Object value) {
				CCSMPre.isTrue(value instanceof String,
						"As we use text cell editor, this should be a string.");
				return validateText((T) element, (String) value);
			}
		});

		return editor;
	}

	/**
	 * Validates the given text. If this is valid, returns null, otherwise a
	 * message describing the error.
	 */
	protected abstract String validateText(T element, String text);

	/** {@inheritDoc}. */
	@SuppressWarnings("unchecked")
	@Override
	protected String getValue(Object element) {
		CCSMPre.isInstanceOf(element, elementClass);
		return getText((T) element);
	}

	/** Returns the textual representation for the given element. */
	protected abstract String getText(T element);

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	protected void setValue(Object element, Object value) {
		// This happens in case of validation problems
		if (value == null) {
			return;
		}

		CCSMPre.isInstanceOf(element, elementClass);
		CCSMPre.isTrue(value instanceof String,
				"As we use text cell editor, this should be a string.");
		String text = (String) value;

		// no change, so ignore
		if (text.equals(getText((T) element))) {
			return;
		}

		setText((T) element, text);
	}

	/** Sets the elements value from the textual representation. */
	protected abstract void setText(T element, String text);

}